@@ -3,7 +3,7 @@ package Assert::Refute::Exec;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = 0.05;
+our $VERSION = 0.06;
 
 =head1 NAME
 
@@ -87,9 +87,9 @@ sub refute {
 
     $msg = $msg ? " - $msg" : '';
     my $n = ++$self->{count};
-    $self->set_result( $n, $cond );
 
     if ($cond) {
+        $self->set_result( $n, $cond );
         $self->do_log( 0, -1, "not ok $n$msg" );
         $self->do_log( 0,  1, $cond ) unless $cond eq 1;
         return 0;
@@ -193,7 +193,20 @@ will be added to this L<Assert::Refute::Exec> by default.
 
 =head3 subcontract( "Message" => $specification, @arguments ... )
 
-Execute a previously defined contract and fail loudly if it fails.
+Execute a previously defined group of tests and fail loudly if it fails.
+
+$specification may be one of:
+
+=over
+
+=item * code reference - will be executed in C<eval> block, with a I<new>
+L<Assert::Refute::Exec> passed as argument;
+
+=item * L<Assert::Refute::Contract> instance - apply() will be called;
+
+=item * L<Assert::Refute::Exec> instance implying a previously executed test.
+
+=back
 
 B<[NOTE]> that the message comes first, unlike in C<refute> or other
 test conditions, and is required.
@@ -201,16 +214,35 @@ test conditions, and is required.
 =cut
 
 sub subcontract {
-    my ($self, $msg, $c, @args) = @_;
+    my ($self, $msg, $sub, @args) = @_;
 
-    $self->_croak("subcontract must be a contract definition or execution log")
-        unless blessed $c;
+    my $rep;
+    if ( blessed $sub and $sub->isa( "Assert::Refute::Contract" ) ) {
+        $rep = $sub->apply(@args);
+    } elsif (blessed $sub and $sub->isa( "Assert::Refute::Exec" ) ) {
+        $self->_croak("pre-executed subcontract cannot take args")
+            if @args;
+        $self->_croak("pre-executed subcontract must be finished")
+            unless $sub->is_done;
+        $rep = $sub;
+    } elsif (UNIVERSAL::isa( $sub, 'CODE' )) {
+        $rep = Assert::Refute::Exec->new;
+        local $Assert::Refute::DRIVER = $rep;
+        eval {
+            $sub->($rep, @args);
+            $rep->done_testing(0);
+            1;
+        } or do {
+            $rep->done_testing( $@ || "Execution interrupted" );
+        };
+    } else {
+        $self->_croak("subcontract must be a contract definition or execution log");
+    };
 
-    my $exec = $c->isa("Assert::Refute::Contract") ? $c->apply(@args) : $c;
-    my $stop = !$exec->is_passing;
+    my $stop = !$rep->is_passing;
     $self->refute( $stop, "$msg (subtest)" );
     if ($stop) {
-        my $log = $exec->get_log;
+        my $log = $rep->get_log;
         $self->do_log( $_->[0]+1, $_->[1], $_->[2] )
             for @$log;
     };
@@ -286,10 +318,12 @@ for failing ones.
 sub get_result {
     my ($self, $n) = @_;
 
-    $self->_croak( "Test $n has never been performed" )
-        unless exists $self->{fail}{$n};
+    return $self->{fail}{$n} || 0
+        if exists $self->{fail}{$n};
 
-    return $self->{fail}{$n} || 0;
+    return 0 if $n =~ /^[1-9]\d*$/ and $n<= $self->{count};
+
+    $self->_croak( "Test $n has never been performed" );
 };
 
 =head3 get_error
@@ -393,6 +427,37 @@ to avoid clash with test names.
 
 This is weird and probably has to be fixed at some point.
 
+=head3 do_run( $code, @list )
+
+Run given CODEREF, passing self as both first argument I<and>
+current_contract().
+Report object is locked afterwards via L</done_testing> call.
+
+Returns self.
+
+Example usage is
+
+    Assert::Refute::Exec->new->run( sub {
+        like $this, qr/.../;
+        can_ok $that, qw(foo bar frobnicate);
+    } );
+
+=cut
+
+sub do_run {
+    my ($self, $code, @args) = @_;
+    local $Assert::Refute::DRIVER = $self;
+    eval {
+        $code->($self, @args);
+        $self->done_testing(0);
+        1;
+    } || do {
+        $self->done_testing(
+            $@ || Carp::shortmess( 'Contract execution interrupted' ) );
+    };
+    return $self;
+};
+
 =head3 do_log( $indent, $level, $message )
 
 Append a message to execution log.
@@ -448,6 +513,8 @@ sub get_log {
 
 Add a refutation to the failed tests log.
 
+This is not guaranteed to be called for passing tests.
+
 =cut
 
 sub set_result {
