@@ -2,7 +2,7 @@ package Assert::Refute::T::Array;
 
 use strict;
 use warnings;
-our $VERSION = 0.05;
+our $VERSION = 0.06;
 
 =head1 NAME
 
@@ -68,35 +68,40 @@ Check that I<every> item in the list matches criteria, which may be one of:
 
 =item * regex - just match against regular expression;
 
-=item * L<Assert::Refute::Contract> - pass list element to this contract as
-argument.
+=item * a functions - execute that function inside a single subcontract;
+
+=item * L<Assert::Refute::Contract> - pass each element as argument to
+a I<separate> subcontract.
 
 =back
 
 =cut
 
-my $is_list = contract {
-    my ($list, $match) = @_;
-
-    if (ref $match eq 'Regexp') {
-        foreach (@$list) {
-            like $_, $match;
-        };
-    } elsif (blessed $match) {
-        foreach (@$list) {
-            subcontract "list item" => $match, $_;
-        };
-    } else {
-        croak "Unknown criterion type: ".(ref $match || 'SCALAR');
-    };
-};
-
-sub array_of ($$;$) { ## no critic
-    my ($list, $match, $message) = @_;
+build_refute array_of => sub {
+    my ($self, $list, $match, $message) = @_;
 
     $message ||= "list of";
-    return subcontract $message => $is_list, $list, $match;
-}
+    $self->subcontract( $message => sub {
+        my $report = shift;
+
+        # TODO 0.30 mention list element number
+        if (ref $match eq 'Regexp') {
+            foreach (@$list) {
+                $report->like( $_, $match );
+            };
+        } elsif (blessed $match && $match->isa("Assert::Refute::Contract")) {
+            foreach (@$list) {
+                $report->subcontract( "list item" => $match, $_ );
+            };
+        } elsif (UNIVERSAL::isa( $match, 'CODE' )) {
+            foreach (@$list) {
+                $match->($report, $_);
+            };
+        } else {
+            croak "array_of: unknown criterion type: ".(ref $match || 'SCALAR');
+        };
+    } ); # end subcontract
+}, export => 1, manual => 1, args => 2;
 
 =head2 is_sorted
 
@@ -117,15 +122,21 @@ build_refute is_sorted => sub {
 
     return '' if @$list < 2;
 
-    my $caller = caller 1;
-    my @bad;
-    my $refa = "${caller}::a";
-    my $refb = "${caller}::b";
+    # Unfortunately, $a and $b are package variables
+    # of the *calling* package...
+    # So localize them through a hack.
+    my ($refa, $refb) = do {
+        my $caller = caller 1;
+        no strict 'refs'; ## no critic
+        \(*{$caller."::a"}, *{$caller."::b"});
+    };
+    local (*$refa, *$refb);
 
+    my @bad;
     for( my $i = 0; $i < @$list - 1; $i++) {
         no strict 'refs'; ## no critic - need to localize caller's $a and $b
-        local $$refa = $list->[$i];
-        local $$refb = $list->[$i+1];
+        *$refa = \$list->[$i];
+        *$refb = \$list->[$i+1];
         $block->() or push @bad, "($i, ".($i+1).")";
     };
 
